@@ -1,58 +1,52 @@
+import logging
 from http import HTTPStatus
-from typing import Iterable, Mapping, Union
+from typing import Iterable, Mapping, Optional, Union
+from flask import current_app as app
 
-from flask_restful import Resource, fields, marshal
+from amundsen_common.models.popular_table import PopularTableSchema
+from amundsen_common.models.user import UserSchema
+from flasgger import swag_from
+from flask_restful import Resource
 
-from metadata_service.api.popular_tables import popular_table_fields
+from metadata_service.api import BaseAPI
 from metadata_service.exception import NotFoundException
 from metadata_service.proxy import get_proxy_client
 from metadata_service.util import UserResourceRel
+from metadata_service.entity.resource_type import to_resource_type
+
+LOGGER = logging.getLogger(__name__)
 
 
-user_detail_fields = {
-    'email': fields.String,
-    'first_name': fields.String,  # Optional
-    'last_name': fields.String,  # Optional
-    'full_name': fields.String,  # Optional
-    'is_active': fields.Boolean,  # Optional
-    'github_username': fields.String,  # Optional
-    'slack_id': fields.String,  # Optional
-    'team_name': fields.String,  # Optional
-    'employee_type': fields.String,  # Optional
-    'manager_fullname': fields.String,  # Optional
-}
-
-table_list_fields = {
-    'table': fields.List(fields.Nested(popular_table_fields))
-}
-
-
-class UserDetailAPI(Resource):
+class UserDetailAPI(BaseAPI):
     """
     User detail API for people resources
     """
 
     def __init__(self) -> None:
         self.client = get_proxy_client()
+        super().__init__(UserSchema, 'user', self.client)
 
-    def get(self, user_id: str) -> Iterable[Union[Mapping, int, None]]:
-        try:
-            table = self.client.get_user_detail(user_id=user_id)
-            return marshal(table, user_detail_fields), HTTPStatus.OK
-
-        except NotFoundException:
-            return {'message': 'User id {} does not exist'.format(user_id)}, HTTPStatus.NOT_FOUND
+    @swag_from('swagger_doc/user/detail_get.yml')
+    def get(self, *, id: Optional[str] = None) -> Iterable[Union[Mapping, int, None]]:
+        if app.config['USER_DETAIL_METHOD']:
+            try:
+                return app.config['USER_DETAIL_METHOD'](id)
+            except Exception:
+                LOGGER.exception('UserDetailAPI GET Failed - Using "USER_DETAIL_METHOD" config variable')
+                return {'message': 'user_id {} fetch failed'.format(id)}, HTTPStatus.NOT_FOUND
+        else:
+            return super().get(id=id)
 
 
-class UserFollowAPI(Resource):
+class UserFollowsAPI(Resource):
     """
-    Build get / put API to support user follow resource features.
-    It will create a relationship(follow / followed_by) between user and resources(table, dashboard etc)
+    Build get API to support user follow resource features.
     """
 
     def __init__(self) -> None:
         self.client = get_proxy_client()
 
+    @swag_from('swagger_doc/user/follow_get.yml')
     def get(self, user_id: str) -> Iterable[Union[Mapping, int, None]]:
         """
         Return a list of resources that user has followed
@@ -63,69 +57,90 @@ class UserFollowAPI(Resource):
         try:
             resources = self.client.get_table_by_user_relation(user_email=user_id,
                                                                relation_type=UserResourceRel.follow)
-            return marshal(resources, table_list_fields), HTTPStatus.OK
+            if len(resources['table']) > 0:
+                return {'table': PopularTableSchema(many=True).dump(resources['table']).data}, HTTPStatus.OK
+            return {'table': []}, HTTPStatus.OK
 
         except NotFoundException:
             return {'message': 'user_id {} does not exist'.format(user_id)}, HTTPStatus.NOT_FOUND
 
         except Exception:
+            LOGGER.exception('UserFollowAPI GET Failed')
             return {'message': 'Internal server error!'}, HTTPStatus.INTERNAL_SERVER_ERROR
 
-    def put(self, user_id: str, resource_type: str, table_uri: str) -> Iterable[Union[Mapping, int, None]]:
-        """
-        Create the follow relationship between user and resources.
-        todo: It will need to refactor all neo4j proxy api to take a type argument.
 
-        :param user_id:
-        :param table_uri:
-        :return:
-        """
-        try:
-            self.client.add_table_relation_by_user(table_uri=table_uri,
-                                                   user_email=user_id,
-                                                   relation_type=UserResourceRel.follow)
-            return {'message': 'The user {} for table_uri {} '
-                               'is added successfully'.format(user_id,
-                                                              table_uri)}, HTTPStatus.OK
-        except Exception as e:
-            return {'message': 'The user {} for table_uri {} '
-                               'is not added successfully'.format(user_id,
-                                                                  table_uri)}, \
-                HTTPStatus.INTERNAL_SERVER_ERROR
-
-    def delete(self, user_id: str, resource_type: str, table_uri: str) -> Iterable[Union[Mapping, int, None]]:
-        """
-        Delete the follow relationship between user and resources.
-        todo: It will need to refactor all neo4j proxy api to take a type argument.
-
-        :param user_id:
-        :param table_uri:
-        :return:
-        """
-        try:
-            self.client.delete_table_relation_by_user(table_uri=table_uri,
-                                                      user_email=user_id,
-                                                      relation_type=UserResourceRel.follow)
-            return {'message': 'The user {} for table_uri {} '
-                               'is added successfully'.format(user_id,
-                                                              table_uri)}, HTTPStatus.OK
-        except Exception as e:
-            return {'message': 'The user {} for table_uri {} '
-                               'is not added successfully'.format(user_id,
-                                                                  table_uri)}, \
-                HTTPStatus.INTERNAL_SERVER_ERROR
-
-
-class UserOwnAPI(Resource):
+class UserFollowAPI(Resource):
     """
-    Build get / put API to support user own resource features.
-    It will create a relationship(owner / owner_of) between user and resources(table, dashboard etc)
-    todo: Deprecate TableOwner API
+    Build put / delete API to support user follow resource features.
+    It will create a relationship(follow / followed_by) between user and resources(table, dashboard etc
     """
 
     def __init__(self) -> None:
         self.client = get_proxy_client()
 
+    @swag_from('swagger_doc/user/follow_put.yml')
+    def put(self, user_id: str, resource_type: str, resource_id: str) -> Iterable[Union[Mapping, int, None]]:
+        """
+        Create the follow relationship between user and resources.
+
+        :param user_id:
+        :param table_uri:
+        :return:
+        """
+        try:
+            self.client.add_resource_relation_by_user(id=resource_id,
+                                                      user_id=user_id,
+                                                      relation_type=UserResourceRel.follow,
+                                                      resource_type=to_resource_type(label=resource_type))
+
+            return {'message': 'The user {} for id {} resource type {} '
+                               'is added successfully'.format(user_id,
+                                                              resource_id,
+                                                              resource_type)}, HTTPStatus.OK
+        except Exception as e:
+            LOGGER.exception('UserFollowAPI PUT Failed')
+            return {'message': 'The user {} for id {} resource type {}'
+                               'is not added successfully'.format(user_id,
+                                                                  resource_id,
+                                                                  resource_type)}, \
+                HTTPStatus.INTERNAL_SERVER_ERROR
+
+    @swag_from('swagger_doc/user/follow_delete.yml')
+    def delete(self, user_id: str, resource_type: str, resource_id: str) -> Iterable[Union[Mapping, int, None]]:
+        """
+        Delete the follow relationship between user and resources.
+
+        :param user_id:
+        :param table_uri:
+        :return:
+        """
+        try:
+            self.client.delete_resource_relation_by_user(id=resource_id,
+                                                         user_id=user_id,
+                                                         relation_type=UserResourceRel.follow,
+                                                         resource_type=to_resource_type(label=resource_type))
+            return {'message': 'The user following {} for id {} resource type {} '
+                               'is deleted successfully'.format(user_id,
+                                                                resource_id,
+                                                                resource_type)}, HTTPStatus.OK
+        except Exception as e:
+            LOGGER.exception('UserFollowAPI DELETE Failed')
+            return {'message': 'The user {} for id {} resource type {} '
+                               'is not deleted successfully'.format(user_id,
+                                                                    resource_id,
+                                                                    resource_type)}, \
+                HTTPStatus.INTERNAL_SERVER_ERROR
+
+
+class UserOwnsAPI(Resource):
+    """
+    Build get API to support user own resource features.
+    """
+
+    def __init__(self) -> None:
+        self.client = get_proxy_client()
+
+    @swag_from('swagger_doc/user/own_get.yml')
     def get(self, user_id: str) -> Iterable[Union[Mapping, int, None]]:
         """
         Return a list of resources that user has owned
@@ -136,14 +151,29 @@ class UserOwnAPI(Resource):
         try:
             resources = self.client.get_table_by_user_relation(user_email=user_id,
                                                                relation_type=UserResourceRel.own)
-            return marshal(resources, table_list_fields), HTTPStatus.OK
+            if len(resources['table']) > 0:
+                return {'table': PopularTableSchema(many=True).dump(resources['table']).data}, HTTPStatus.OK
+            return {'table': []}, HTTPStatus.OK
 
         except NotFoundException:
             return {'message': 'user_id {} does not exist'.format(user_id)}, HTTPStatus.NOT_FOUND
 
         except Exception:
+            LOGGER.exception('UserOwnAPI GET Failed')
             return {'message': 'Internal server error!'}, HTTPStatus.INTERNAL_SERVER_ERROR
 
+
+class UserOwnAPI(Resource):
+    """
+    Build put / delete API to support user own resource features.
+    It will create a relationship(owner / owner_of) between user and resources(table, dashboard etc)
+    todo: Deprecate TableOwner API
+    """
+
+    def __init__(self) -> None:
+        self.client = get_proxy_client()
+
+    @swag_from('swagger_doc/user/own_put.yml')
     def put(self, user_id: str, resource_type: str, table_uri: str) -> Iterable[Union[Mapping, int, None]]:
         """
         Create the follow relationship between user and resources.
@@ -159,10 +189,12 @@ class UserOwnAPI(Resource):
                                'is added successfully'.format(user_id,
                                                               table_uri)}, HTTPStatus.OK
         except Exception as e:
+            LOGGER.exception('UserOwnAPI PUT Failed')
             return {'message': 'The owner {} for table_uri {} '
                                'is not added successfully'.format(user_id,
                                                                   table_uri)}, HTTPStatus.INTERNAL_SERVER_ERROR
 
+    @swag_from('swagger_doc/user/own_delete.yml')
     def delete(self, user_id: str, resource_type: str, table_uri: str) -> Iterable[Union[Mapping, int, None]]:
         try:
             self.client.delete_owner(table_uri=table_uri, owner=user_id)
@@ -170,20 +202,21 @@ class UserOwnAPI(Resource):
                                'is deleted successfully'.format(user_id,
                                                                 table_uri)}, HTTPStatus.OK
         except Exception:
+            LOGGER.exception('UserOwnAPI DELETE Failed')
             return {'message': 'The owner {} for table_uri {} '
                                'is not deleted successfully'.format(user_id,
                                                                     table_uri)}, HTTPStatus.INTERNAL_SERVER_ERROR
 
 
-class UserReadAPI(Resource):
+class UserReadsAPI(Resource):
     """
-    Build get / put API to support user read resource features.
-    It will create a relationship(read / read_by) between user and resources(table, dashboard etc)
+    Build get API to support user read resource features.
     """
 
     def __init__(self) -> None:
         self.client = get_proxy_client()
 
+    @swag_from('swagger_doc/user/read_get.yml')
     def get(self, user_id: str) -> Iterable[Union[Mapping, int, None]]:
         """
         Return a list of resources that user has read
@@ -192,12 +225,14 @@ class UserReadAPI(Resource):
         :return:
         """
         try:
-            resources = self.client.get_table_by_user_relation(user_email=user_id,
-                                                               relation_type=UserResourceRel.read)
-            return marshal(resources, table_list_fields), HTTPStatus.OK
+            resources = self.client.get_frequently_used_tables(user_email=user_id)
+            if len(resources['table']) > 0:
+                return {'table': PopularTableSchema(many=True).dump(resources['table']).data}, HTTPStatus.OK
+            return {'table': []}, HTTPStatus.OK
 
         except NotFoundException:
             return {'message': 'user_id {} does not exist'.format(user_id)}, HTTPStatus.NOT_FOUND
 
         except Exception:
+            LOGGER.exception('UserReadsAPI GET Failed')
             return {'message': 'Internal server error!'}, HTTPStatus.INTERNAL_SERVER_ERROR
